<?php
/**
 * Diese Datei ist Teil von „Lernstrategien“ 
 * (Dateiname: „mod_splernstrategien“), einer Plugin-Software 
 * für das moodle LMS.
 *
 * Das moodle LMS ist lizenziert mit der GNU General Public 
 * License Version 3. Aktuelle Lizenzhinweise zu moodle 
 * finden Sie unter: https://moodledev.io/general/license.
 *
 * Das Plugin ist lizenziert mit der GNU General Public 
 * License Version 3. Anzugebender Urheber des Plugins ist
 * die „metromorph softworks GmbH im Auftrag von ORCA.nrw“.
 * Die Lizenzierung umfasst nicht die Inhalte, die auf der
 * Anwenderoberfläche des Plugins angezeigt werden; diese sind 
 * ihrerseits CC BY-SA 4.0 lizenziert.
 *
 * Sie sollten eine Kopie der GNU-Lizenz zusammen mit diesem 
 * Programm erhalten haben. Sie finden sie auch unter 
 * https://www.gnu.org/licenses. Die CC-Lizenz finden Sie
 * unter https://creativecommons.org/licenses/by-sa/4.0/deed.de.
 */


/**
 * Implementation for activity-completion.
 *
 * @package   mod_splernstrategien
 * @category  completion
 * @license   https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

declare(strict_types=1);

namespace mod_splernstrategien\completion;

use CompileError;
use core_completion\activity_custom_completion;

/**
 * Activity custom completion subclass for the splernstrategien activity.
 *
 * Class for defining mod_splernstrategien's custom completion rules and fetching the completion statuses
 * of the custom completion rules for a given instance and a user.
 *
 * @package   mod_splernstrategien
 * @copyright metromorph softworks devel@metromorph.de
 * @license   https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class custom_completion extends activity_custom_completion
{

    /**
     * Fetches the completion state for a given completion rule.
     *
     * @param string $rule The completion rule.
     * @return int The completion state.
     */
    public function get_state(string $rule): int
    {
        global $DB;

        $state = $DB->get_record(
            'splernstrategien_users',
            ['moodle_id' => $this->cm->id, 'user_id' => $this->userid],
            'laststate,epa_finished,epb_finished,wp_finished',
            IGNORE_MISSING
        );

        $completed = false;

        if (isset($state->laststate)) {
            //we have to repair that some states are not ordered
            switch ($state->laststate) {
                case 521:
                case 522:
                    $state->laststate = 52;
                    break;
                case 1132:
                    $state->laststate = 132;
                    break;
                case 1260:
                    $state->laststate = 260;
                    break;
                case 1388:
                    $state->laststate = 388;
                    break;
            }

            $this->validate_rule($rule);

            switch ($rule) {
                case 'intro':
                    if ($state->laststate >= 15) {
                        $completed = true;
                    }
                    break;
                case 'base_epa':
                    if ($state->laststate >= 15 && $state->epa_finished == "1") {
                        $completed = true;
                    }
                    break;
                case  'base_epb':
                    if ($state->laststate >= 15 && $state->epb_finished == "1") {
                        $completed = true;
                    }
                    break;
                case  'base_wp':
                    if ($state->laststate >= 15 && $state->wp_finished == "1") {
                        $completed = true;
                    }
                    break;
                case  'base_finish':
                    if ($state->laststate >= 127) {
                        $completed = true;
                    }
                    break;
                case  'trigger_1':
                    if ($state->laststate >= 255) {
                        $completed = true;
                    }
                    break;
                case  'trigger_2':
                    if ($state->laststate >= 383) {
                        $completed = true;
                    }
                    break;
                case  'trigger_3':
                    if ($state->laststate >= 397) {
                        $completed = true;
                    }
                    break;

                default:
                    $completed = false;
                    break;
            }
        }


        return $completed ? COMPLETION_COMPLETE : COMPLETION_INCOMPLETE;
    }

    /**
     * Fetch the list of custom completion rules that this module defines.
     *
     * @return array
     */
    public static function get_defined_custom_rules(): array
    {
        return [
            'intro',
            'base_epa',
            'base_epb',
            'base_wp',
            'base_finish',
            'trigger_1',
            'trigger_2',
            'trigger_3',
        ];
    }

    /**
     * Returns an associative array of the descriptions of custom completion rules.
     *
     * @return array
     */
    public function get_custom_rule_descriptions(): array
    {
        return [
            'intro'       => get_string('completiondetail:intro', 'splernstrategien'),
            'base_epa'    => get_string('completiondetail:base_epa', 'splernstrategien'),
            'base_epb'    => get_string('completiondetail:base_epb', 'splernstrategien'),
            'base_wp'     => get_string('completiondetail:base_wp', 'splernstrategien'),
            'base_finish' => get_string('completiondetail:base_finish', 'splernstrategien'),
            'trigger_1'   => get_string('completiondetail:trigger_1', 'splernstrategien'),
            'trigger_2'   => get_string('completiondetail:trigger_2', 'splernstrategien'),
            'trigger_3'   => get_string('completiondetail:trigger_3', 'splernstrategien'),
        ];
    }

    /**
     * Returns an array of all completion rules, in the order they should be displayed to users.
     *
     * @return array
     */
    public function get_sort_order(): array
    {
        return [
            'intro',
            'base_epa',
            'base_epb',
            'base_wp',
            'base_finish',
            'trigger_1',
            'trigger_2',
            'trigger_3',
        ];
    }
}
